@@ -23,7 +23,8 @@ import json
 import os
 import sys
 
-from .models import load_model, quantize, dump_config, save_weights, load_weights
+from .models import (load_model, quantize, dump_config, save_weights, load_weights,
+                     check_quantization)
 from .models.transforms import fold_rescaling
 
 
@@ -121,6 +122,18 @@ def load_model_weights(model_path, weights_path):
     print(f"Saved model with new weights to {model_path}.")
 
 
+def check_quantized_model(model_path):
+    """ CLI entry point to check model quantization.
+
+    Args:
+        model_path (str): Path to the model on which apply the weights.
+    """
+    model = load_model(model_path)
+    messages = check_quantization(model)
+    for msg in messages:
+        print(msg)
+
+
 def main():
     """ CLI entry point.
 
@@ -168,6 +181,12 @@ def main():
     w_parser.add_argument("-w", "--weights_path", type=str, required=True,
                           help="Npz file that contains the weights to apply.")
 
+    # Check action and arguments
+    c_parser = sp.add_parser(
+        "check", help="Check the quantization of a model to detect issues that can be fixed "
+        "by adjusting the quantization configuration.")
+    c_parser.add_argument("-m", "--model", type=str, required=True, help="Model to check")
+
     args = parser.parse_args()
 
     if args.action == "version":
@@ -202,3 +221,5 @@ def main():
             model_path=args.model,
             weights_path=args.weights_path,
         )
+    elif args.action == "check":
+        check_quantized_model(args.model)
