@@ -20,7 +20,8 @@ import tensorflow as tf
 from ..tensors import FixedPoint, MAX_BUFFER_BITWIDTH, round_log2, pow2
 from .reshaping import QuantizedReshape, QuantizedPermute
 from .shiftmax import shiftmax, QuantizedShiftmax
-from .layers import deserialize_quant_object, Calibrable, CalibrableVariable
+from .calibrable import Calibrable, CalibrableVariable
+from .quantizers import deserialize_quantizer, OutputQuantizer
 
 
 __all__ = ["Attention", "string_to_softmax", "QuantizedAttention"]
@@ -85,7 +86,6 @@ class Attention(keras.layers.Layer):
         output: [batch, num_heads, token, dim] -> [batch, token, hidden]
 
     Args:
-
         num_heads (int): the number of attention heads
         softmax (str, optional): 'softmax' or 'shiftmax'. Defaults to 'softmax'
 
@@ -151,6 +151,14 @@ class Attention(keras.layers.Layer):
 
 @tf.keras.utils.register_keras_serializable()
 class QuantizedAttention(Calibrable, Attention):
+    """An Attention layer that operates on quantized inputs
+
+    Args:
+        num_heads (int): the number of attention heads
+        softmax (str, optional): 'softmax' or 'shiftmax'. Defaults to 'shiftmax'
+        quant_config (dict, optional): the serialized quantization configuration.
+            Defaults to empty configuration.
+    """
 
     def __init__(self, num_heads, softmax='shiftmax', quant_config={}, **kwargs):
         if softmax != 'shiftmax':
@@ -159,8 +167,8 @@ class QuantizedAttention(Calibrable, Attention):
         super().__init__(num_heads=num_heads, softmax=softmax, **kwargs)
         self.quant_config = quant_config
 
-        self.out_quantizer = deserialize_quant_object(
-            self.quant_config, "output_quantizer", False)
+        self.out_quantizer = deserialize_quantizer(
+            self.quant_config, "output_quantizer", OutputQuantizer, False)
 
         # Override softmax operation
         softmax_quant_conf = self.quant_config.get("softmax", {})
