@@ -19,7 +19,8 @@ import tensorflow as tf
 from keras import layers, backend
 from keras.utils import conv_utils
 
-from .layers import deserialize_quant_object, Calibrable, CalibrableVariable
+from .calibrable import Calibrable, CalibrableVariable
+from .quantizers import deserialize_quantizer, WeightQuantizer, OutputQuantizer
 from ..tensors import FixedPoint, MAX_BUFFER_BITWIDTH, QFloat
 
 
@@ -125,13 +126,13 @@ class QuantizedConv2D(Calibrable, layers.Conv2D):
     def __init__(self, *args, padding_value=None, quant_config={}, **kwargs):
         super().__init__(*args, **kwargs)
         self.quant_config = quant_config
-        self.out_quantizer = deserialize_quant_object(
-            self.quant_config, "output_quantizer", False)
-        self.weight_quantizer = deserialize_quant_object(
-            self.quant_config, "weight_quantizer", True)
+        self.out_quantizer = deserialize_quantizer(
+            self.quant_config, "output_quantizer", OutputQuantizer, False)
+        self.weight_quantizer = deserialize_quantizer(
+            self.quant_config, "weight_quantizer", WeightQuantizer, True)
         if self.use_bias:
-            self.bias_quantizer = deserialize_quant_object(
-                self.quant_config, "bias_quantizer", True)
+            self.bias_quantizer = deserialize_quantizer(
+                self.quant_config, "bias_quantizer", WeightQuantizer, True)
         self.buffer_bitwidth = self.quant_config.get(
             "buffer_bitwidth", MAX_BUFFER_BITWIDTH) - 1
         assert self.buffer_bitwidth > 0, "The buffer_bitwidth must be a strictly positive integer."
@@ -140,8 +141,8 @@ class QuantizedConv2D(Calibrable, layers.Conv2D):
             if tf.size(padding_value) != 1:
                 raise ValueError(f"The padding value must be a scalar. Receives {padding_value}.")
             padding_value = FixedPoint.quantize(padding_value,
-                                                self._PADDING_FRAC_BITS,
-                                                self.buffer_bitwidth)
+                                                self.buffer_bitwidth,
+                                                self._PADDING_FRAC_BITS)
             padding_value = padding_value.promote(self.buffer_bitwidth)
             # _padding_value is a private member because value controls are operated
             # in the constructor.
@@ -152,6 +153,14 @@ class QuantizedConv2D(Calibrable, layers.Conv2D):
         else:
             self._padding_value = None
 
+        # Add objects that will store the shift values.
+        self.input_shift = CalibrableVariable()
+        if self._padding_value is not None:
+            self.pad_shift = CalibrableVariable()
+        if self.use_bias:
+            self.conv_shift = CalibrableVariable()
+            self.bias_shift = CalibrableVariable()
+
     def call(self, inputs, training=None):
         # raise an error if the inputs are not FixedPoint or tf.Tensor
         if not isinstance(inputs, (FixedPoint, tf.Tensor)):
@@ -164,15 +173,17 @@ class QuantizedConv2D(Calibrable, layers.Conv2D):
         if isinstance(inputs, tf.Tensor):
             # Assume the inputs are integer stored as float, which is the only tf.Tensor
             # inputs that are allowed
-            inputs = FixedPoint.quantize(inputs, 0, 8)
+            inputs = FixedPoint(inputs, 0, 8)
 
         # Promote inputs to avoid saturation, then align them
-        inputs, _ = inputs.promote(self.buffer_bitwidth).align()
+        inputs, shift = inputs.promote(self.buffer_bitwidth).align()
+        self.input_shift(shift)
 
         # We need a custom padding for specifics padding values
         if self._padding_value is not None:
             # Set the same scale factor as inputs
-            padding_value, _ = self._padding_value.align(inputs)
+            padding_value, shift = self._padding_value.align(inputs)
+            self.pad_shift(shift)
 
             # Note that we use the custom padding even when padding value is 0.
             inputs_padded = apply_padding(inputs.values,
@@ -198,8 +209,10 @@ class QuantizedConv2D(Calibrable, layers.Conv2D):
             bias = self.bias_quantizer(self.bias, training)
 
             # Align intermediate outputs and biases before adding them
-            outputs, _ = outputs.align(bias)
-            bias, _ = bias.promote(self.buffer_bitwidth).align(outputs)
+            outputs, shift = outputs.align(bias)
+            self.conv_shift(shift)
+            bias, shift = bias.promote(self.buffer_bitwidth).align(outputs)
+            self.bias_shift(shift)
             outputs = tf.add(outputs, bias)
 
         if self.out_quantizer is not None:
@@ -226,12 +239,13 @@ class QuantizedConv2DTranspose(Calibrable, layers.Conv2DTranspose):
     def __init__(self, *args, quant_config={}, **kwargs):
         super().__init__(*args, **kwargs)
         self.quant_config = quant_config
-        self.out_quantizer = deserialize_quant_object(self.quant_config, "output_quantizer", False)
-        self.weight_quantizer = deserialize_quant_object(self.quant_config, "weight_quantizer",
-                                                         True)
+        self.out_quantizer = deserialize_quantizer(
+            self.quant_config, "output_quantizer", OutputQuantizer, False)
+        self.weight_quantizer = deserialize_quantizer(self.quant_config, "weight_quantizer",
+                                                      WeightQuantizer, True)
         if self.use_bias:
-            self.bias_quantizer = deserialize_quant_object(self.quant_config, "bias_quantizer",
-                                                           True)
+            self.bias_quantizer = deserialize_quantizer(self.quant_config, "bias_quantizer",
+                                                        WeightQuantizer, True)
         self.buffer_bitwidth = self.quant_config.get("buffer_bitwidth", MAX_BUFFER_BITWIDTH) - 1
         assert self.buffer_bitwidth > 0, "The buffer_bitwidth must be a strictly positive integer."
 
