@@ -23,7 +23,8 @@ from keras import backend
 from keras.layers import DepthwiseConv2D
 
 from ..tensors import FixedPoint, MAX_BUFFER_BITWIDTH
-from .layers import deserialize_quant_object, Calibrable, CalibrableVariable
+from .calibrable import Calibrable, CalibrableVariable
+from .quantizers import deserialize_quantizer, WeightQuantizer, OutputQuantizer
 
 
 __all__ = ["QuantizedDepthwiseConv2D"]
@@ -32,20 +33,24 @@ __all__ = ["QuantizedDepthwiseConv2D"]
 @tf.keras.utils.register_keras_serializable()
 class QuantizedDepthwiseConv2D(Calibrable, DepthwiseConv2D):
     """ A depthwise convolutional layer that operates on quantized inputs and weights.
+
+    Args:
+        quant_config (dict, optional): the serialized quantization configuration.
+            Defaults to empty configuration.
     """
 
     def __init__(self, *args, quant_config={}, **kwargs):
         super().__init__(*args, **kwargs)
         self.quant_config = quant_config
 
-        self.out_quantizer = deserialize_quant_object(
-            self.quant_config, "output_quantizer", False)
+        self.out_quantizer = deserialize_quantizer(
+            self.quant_config, "output_quantizer", OutputQuantizer, False)
 
-        self.weight_quantizer = deserialize_quant_object(
-            self.quant_config, "weight_quantizer", True)
+        self.weight_quantizer = deserialize_quantizer(
+            self.quant_config, "weight_quantizer", WeightQuantizer, True)
         if self.use_bias:
-            self.bias_quantizer = deserialize_quant_object(
-                self.quant_config, "bias_quantizer", True)
+            self.bias_quantizer = deserialize_quantizer(
+                self.quant_config, "bias_quantizer", WeightQuantizer, True)
 
         self.buffer_bitwidth = self.quant_config.get("buffer_bitwidth", MAX_BUFFER_BITWIDTH) - 1
         assert self.buffer_bitwidth > 0, "The buffer_bitwidth must be a strictly positive integer."
@@ -68,7 +73,7 @@ class QuantizedDepthwiseConv2D(Calibrable, DepthwiseConv2D):
         if isinstance(inputs, tf.Tensor):
             # Assume the inputs are integer stored as float, which is the only tf.Tensor
             # inputs that are allowed
-            inputs = FixedPoint.quantize(inputs, 0, 8)
+            inputs = FixedPoint(inputs, 0, 8)
 
         # Promote inputs to avoid saturation, then align them
         inputs, shift = inputs.promote(self.buffer_bitwidth).align()
