@@ -25,6 +25,10 @@ __all__ = ["QDropout"]
 
 @tf.keras.utils.register_keras_serializable()
 class QDropout(keras.layers.Dropout):
+    """ A dropout layer that operates on quantized inputs and weights.
+
+    It is only implemented as a passthrough.
+    """
 
     def call(self, inputs, training=None):
         # raise an error if the inputs are not FixedPoint
