@@ -18,8 +18,9 @@ import tensorflow as tf
 import keras
 from tensorflow.python.framework import ops
 
-from .layers import deserialize_quant_object, Calibrable, CalibrableVariable
-from ..tensors import FixedPoint, reciprocal_lut, MAX_BUFFER_BITWIDTH
+from .calibrable import Calibrable, CalibrableVariable
+from .quantizers import deserialize_quantizer, WeightQuantizer, OutputQuantizer
+from ..tensors import FixedPoint, MAX_BUFFER_BITWIDTH
 
 
 __all__ = ["QuantizedLayerNormalization", "LayerMadNormalization"]
@@ -27,28 +28,23 @@ __all__ = ["QuantizedLayerNormalization", "LayerMadNormalization"]
 
 @tf.keras.utils.register_keras_serializable()
 class LayerMadNormalization(keras.layers.LayerNormalization):
-    """Approximates the :obj:`keras.layers.LayerNormalization` (LN), replacing the computation of
+    r"""Approximates the :obj:`keras.layers.LayerNormalization` (LN), replacing the computation of
     the standard deviation by the mean average deviation (mad).
 
     Taking into account the complexity of the calculate required in the standard deviation,
-    the LayerMadNormalization (LMN) is intended to replace the :math:`std(x)` by :math:`mad(x)`,
-    defined as:
+    the LayerMadNormalization (LMN) is intended to replace the :math:`std(x)` by :math:`mad(x)`
+    on a Power of Two (PoT), defined as:
 
-    .. math::
-        mad(x) = |x - mean(x)|
+    .. math:: mad(x) = |x - mean(x)| * 2^\text{-nb_channels}
+
+    To simplify it even more and make it more hardware-friendly, :math:`mean(x)` is considered
+    as 0, so. These are the resulting calculations:
+
+    .. math:: mad(x) = sum(|x|) * 2^\text{-nb_channels}
 
     Then, the equation of the layer is defined as:
 
-    .. math::
-        LMN(x) = \gamma(x - mean(x))/mad(x) + \beta
-
-    Also, to develop a hardware compatible quantization, we make the approximation of mean
-    on a Power Of Two (PoT):
-
-    .. math::
-        mean(x) = sum(x) * 2**-nb_channels
-
-    Where :math:`nb_channels = round(log2(inputs.shape[axis]))`
+    .. math:: LMN(x) = \gamma\frac{x}{mad(x)} + \beta
 
     .. note::
         A tuning step in the switching procedure between the LN to LMN layer will be required
@@ -58,42 +54,39 @@ class LayerMadNormalization(keras.layers.LayerNormalization):
     def build(self, input_shape):
         super().build(input_shape)
 
-        # Save an approximation of nb_channels as PoT to calculate the mean of inputs as:
-        # mean_inputs = sum(inputs) * 2 ** -mean_shift
+        # Save an approximation of nb_channels as PoT to calculate the mad of inputs as:
+        # mad = sum(abs(inputs)) * 2 ** -mad_shift
         if len(self.axis) > 1:
             raise ValueError(f"{self.__class__.__name__} does not support multiple axis.")
         nb_channels = tf.cast(input_shape[self.axis[0]], tf.float32)
-        self.mean_shift = tf.math.round(tf.math.log(nb_channels) / tf.math.log(2.))
+        self.mad_shift = tf.math.round(tf.math.log(nb_channels) / tf.math.log(2.))
 
-    def _moments(self, inputs, axes, *args, name=None, **kwargs):
-        """Compute the mean and mean absolute deviation (mad).
+    def _get_mad(self, inputs, axis, *args, name=None, **kwargs):
+        """Compute the mean absolute deviation (mad).
 
         Args:
             inputs (:obj:`tensorflow.Tensor`): input tensor.
-            axes (array of ints): axes along which to compute mean and mad.
-            name (str, optional): name used to scope the operations that compute the moments.
+            axis (array of ints): axis along which to compute mad.
+            name (str, optional): name used to scope the operations that compute the mad.
                 Defaults to None.
-            *args, **kwargs (dict, optional): additional arguments of :func:`tf.math.reduce_mean`.
+            *args, **kwargs (dict, optional): additional arguments of :func:`tf.math.reduce_sum`.
 
         Returns:
-            :obj:`tensorflow.Tensor`, :obj:`tensorflow.Tensor`: mean around the axis and
-                mean absolute deviation (mad).
+            :obj:`tensorflow.Tensor`: mean absolute deviation (mad).
         """
-        # Compute the mean and mad, taking into account the approximation:
+        # Compute the mad, taking into account the approximations:
+        # mad \approx sum(abs(inputs))
         # nb_channels \approx 2 ** round(log2(inputs.shape[axis]))
-        nb_channels_inv = 2.0 ** -self.mean_shift
-        with ops.name_scope(name, "_moments", [inputs, axes]):
-            mean = tf.math.reduce_sum(
-                inputs, axes, *args, **kwargs, name="mean")
-            mean = mean * nb_channels_inv
-            mad = tf.math.reduce_sum(tf.math.abs(
-                inputs - mean), axes, *args, **kwargs, name="mad")
+        nb_channels_inv = 2.0 ** -self.mad_shift
+        with ops.name_scope(name, "_mad", [inputs, axis]):
+            mad = tf.math.reduce_sum(tf.math.abs(inputs), axis, *args, **kwargs, name="mad")
             mad = mad * nb_channels_inv
-        return mean, mad
+        return mad
 
     def call(self, inputs, training=None):
-        """Use the same call as the parent, changing tf.nn.moments instead of self._moments."""
-        # Compute the axis along which to reduce the mean / mad
+        """Use the same call as the parent, changing tf.nn.moments instead of self._get_mad,
+        and replacing mean by 0"""
+        # Compute the axis along which to reduce the mad
         input_shape = inputs.shape
         ndims = len(input_shape)
 
@@ -113,12 +106,12 @@ class LayerMadNormalization(keras.layers.LayerNormalization):
             # least as numerically stable as the fused version.
             inputs = tf.cast(inputs, 'float32')
 
-        # Calculate the moments on the last axis (layer activations).
-        mean, mad = self._moments(inputs, self.axis, keepdims=True)
+        # Calculate the mad on the last axis (layer activations).
+        mad = self._get_mad(inputs, self.axis, keepdims=True)
         scale, offset = _broadcast(self.gamma), _broadcast(self.beta)
 
         # Compute layer normalization as layer mad normalization
-        outputs = scale * (inputs - mean) / (mad + self.epsilon) + offset
+        outputs = scale * inputs / mad + offset
         outputs = tf.cast(outputs, input_dtype)
 
         # If some components of the shape got lost due to adjustments, fix that.
@@ -129,27 +122,31 @@ class LayerMadNormalization(keras.layers.LayerNormalization):
 
 @tf.keras.utils.register_keras_serializable()
 class QuantizedLayerNormalization(Calibrable, LayerMadNormalization):
+    """ A LayerNormalization layer that operates on quantized inputs and weights.
 
+     Args:
+        quant_config (dict, optional): the serialized quantization configuration.
+            Defaults to empty configuration.
+    """
     def __init__(self, *args, quant_config={}, **kwargs):
         super().__init__(*args, **kwargs)
         self.quant_config = quant_config
-        self.out_quantizer = deserialize_quant_object(
-            self.quant_config, "output_quantizer", False)
+        self.out_quantizer = deserialize_quantizer(
+            self.quant_config, "output_quantizer", OutputQuantizer, False)
         self.buffer_bitwidth = self.quant_config.get(
             "buffer_bitwidth", MAX_BUFFER_BITWIDTH) - 1
         self.mad_rec_frac_bits = self.quant_config.get("mad_rec_frac_bits", 16)
         assert self.buffer_bitwidth > 0, "The buffer_bitwidth must be a strictly positive integer."
         assert self.mad_rec_frac_bits >= 0, "The mad_rec_frac_bits must be a non-negative integer."
-        self.gamma_quantizer = deserialize_quant_object(
-            self.quant_config, "gamma_quantizer", True)
-        self.beta_quantizer = deserialize_quant_object(
-            self.quant_config, "beta_quantizer", True)
-        self.mad_quantizer = deserialize_quant_object(
-            self.quant_config, "mad_quantizer", True)
+        self.gamma_quantizer = deserialize_quantizer(
+            self.quant_config, "gamma_quantizer", WeightQuantizer, True)
+        self.beta_quantizer = deserialize_quantizer(
+            self.quant_config, "beta_quantizer", WeightQuantizer, True)
 
         # Add objects that will store the shift values.
         self.input_shift = CalibrableVariable()
         self.x_rescaled_shift = CalibrableVariable()
+        self.one_bits = CalibrableVariable()
         self.beta_shift = CalibrableVariable()
         self.output_shift = CalibrableVariable()
 
@@ -173,42 +170,29 @@ class QuantizedLayerNormalization(Calibrable, LayerMadNormalization):
         inputs, shift = inputs.promote(self.buffer_bitwidth).align()
         self.input_shift(shift)
 
-        # Compute the Mean of the inputs
-        sum_x = tf.math.reduce_sum(inputs, self.axis, keepdims=True)
-        # Approximate mean_x = sum_x / 2**mean_shift
-        # We could apply the shift virtually, but at this stage it is ok to drop the precision
-        # on the mean as it does not change the accuracy of the operation and prevents
-        # the explosion of the frac_bits.
-        # From an hardware perspective, it will correspond to an unconditional right-shift.
-        mean_x = sum_x.shift(-self.mean_shift)
-
         # Compute the Absolute Deviation (MAD)
-        x_shifted = inputs - mean_x
         mad_sum = tf.math.reduce_sum(
-            x_shifted.abs(), self.axis, keepdims=True, name="mad")
-        # Approximate mad = mad_sum / 2**mean_shift
-        mad = mad_sum >> self.mean_shift
-
-        mad = self.mad_quantizer(mad, training=training)
-
-        # Convert the learnable parameters gamma and beta in FixedPoint
-        gamma = self.gamma_quantizer(self.gamma, training=training)
+            inputs.abs(), self.axis, keepdims=True, name="mad")
+        # Calculate a gamma_rescaled = gamma * 2 ** mad_shift. This is the
+        # value that will be quantized.
+        gamma_rescaled = self.gamma * (2 ** self.mad_shift)
+        # Quantize the learnable parameters gamma_rescaled and beta
+        gamma_rescaled = self.gamma_quantizer(
+            gamma_rescaled, training=training)
         beta = self.beta_quantizer(self.beta, training=training)
 
-        # Find the FixedPoint Reciprocal of the FixedPoint mad
-        mad_inv = reciprocal_lut(
-            mad, self.mad_rec_frac_bits, self.buffer_bitwidth)
+        # Calculate the gamma_on_mad_sum = (gamma * 2**mad_shift) / mad_sum.
+        # This can be evaluated once on hardware.
+        gamma_on_mad_sum = gamma_rescaled / mad_sum
 
-        # Rescale inputs
-        x_rescaled = x_shifted * mad_inv
+        # Multiply inputs by gamma
+        x_rescaled = inputs * gamma_on_mad_sum
 
         # Downscale to recover the initial fractional bits
-        x_rescaled, shift = x_rescaled.downscale(x_shifted.frac_bits, self.buffer_bitwidth)
+        outputs, shift = x_rescaled.downscale(
+            inputs.frac_bits, self.buffer_bitwidth)
         self.x_rescaled_shift(shift)
 
-        # Multiply by gamma
-        outputs = x_rescaled * gamma
-
         # Before adding the bias, we need aligned results
         outputs, shift = outputs.align(beta)
         self.output_shift(shift)
