@@ -18,7 +18,8 @@ import tensorflow as tf
 
 from keras.layers import MaxPool2D, GlobalAveragePooling2D
 
-from . import Calibrable, deserialize_quant_object, CalibrableVariable
+from .calibrable import Calibrable, CalibrableVariable
+from .quantizers import deserialize_quantizer, OutputQuantizer
 from ..tensors import FixedPoint, MAX_BUFFER_BITWIDTH
 
 __all__ = ["QuantizedMaxPool2D", "QuantizedGlobalAveragePooling2D"]
@@ -44,12 +45,18 @@ class QuantizedMaxPool2D(MaxPool2D):
 class QuantizedGlobalAveragePooling2D(Calibrable, GlobalAveragePooling2D):
     """A global average pooling layer that operates on quantized inputs.
 
+     Args:
+        quant_config (dict, optional): the serialized quantization configuration.
+            Defaults to empty configuration.
     """
 
     def __init__(self, quant_config={}, **kwargs):
         super().__init__(**kwargs)
         self.quant_config = quant_config
-        self.out_quantizer = deserialize_quant_object(self.quant_config, "output_quantizer", False)
+        self.out_quantizer = deserialize_quantizer(
+            self.quant_config, "output_quantizer", OutputQuantizer, False)
+        self.buffer_bitwidth = self.quant_config.get("buffer_bitwidth", MAX_BUFFER_BITWIDTH) - 1
+        assert self.buffer_bitwidth > 0, "The buffer_bitwidth must be a strictly positive integer."
 
         # Add objects that will store the shift values.
         self.input_shift = CalibrableVariable()
@@ -66,7 +73,7 @@ class QuantizedGlobalAveragePooling2D(Calibrable, GlobalAveragePooling2D):
                              Receives {type(inputs)} inputs.")
 
         # Promote input to prevent overflow in the reduce_sum op, then align them
-        inputs, shift = inputs.promote(MAX_BUFFER_BITWIDTH).align()
+        inputs, shift = inputs.promote(self.buffer_bitwidth).align()
         self.input_shift(shift)
         inputs_sum = tf.reduce_sum(inputs, axis=[1, 2], keepdims=self.keepdims)
         inputs_mean = inputs_sum / self.spatial_size
