@@ -18,7 +18,8 @@ import keras
 import tensorflow as tf
 
 from ..tensors import ceil_log2, FixedPoint, MAX_BUFFER_BITWIDTH
-from .layers import deserialize_quant_object, Calibrable, CalibrableVariable
+from .calibrable import Calibrable, CalibrableVariable
+from .quantizers import deserialize_quantizer, OutputQuantizer
 from ..debugging import assert_less_equal
 
 
@@ -66,27 +67,28 @@ class QuantizedReciprocal(Calibrable, Reciprocal):
         super().__init__(*args, **kwargs)
         self.quant_config = quant_config
         self.buffer_bitwidth = quant_config.get("buffer_bitwidth", MAX_BUFFER_BITWIDTH) - 1
-        self.out_quantizer = deserialize_quant_object(quant_config, "output_quantizer", False)
+        self.out_quantizer = deserialize_quantizer(
+            quant_config, "output_quantizer", OutputQuantizer, False)
 
         # All constants are in the range |x| < 2. We only need 1 bit for the int part.
         # However, two approaches are considered for the fractional part:
         # 1. 'Multiplying' constants will have the smallest possible frac_bits (4)
         mul_frac_bits = mul_value_bits = 4
-        self._K_0_3125 = FixedPoint.quantize(-0.3125, mul_frac_bits, mul_value_bits)
-        self._K_0_625 = FixedPoint.quantize(-0.625, mul_frac_bits, mul_value_bits)
+        self._K_0_3125 = FixedPoint.quantize(-0.3125, mul_value_bits, mul_frac_bits)
+        self._K_0_625 = FixedPoint.quantize(-0.625, mul_value_bits, mul_frac_bits)
         # 2. 'Adding' constant will be aligned with the result of x' = c*x (c as constant)
         #    Note: it is possible to calculate a-priori the alignment, since algorithm ensures
         #    that x'.frac_bits = x.frac_bits + c.frac_bits = (self.buffer_bitwidth - 1 - 4) + 4
         #                      = self.buffer_bitwidth - 1
         value_bits = self.buffer_bitwidth
         frac_bits = value_bits - 1
-        self._K_1_125 = FixedPoint.quantize(1.125, frac_bits, value_bits)
-        self._K_1_59375 = FixedPoint.quantize(1.59375, frac_bits, value_bits)
+        self._K_1_125 = FixedPoint.quantize(1.125, value_bits, frac_bits)
+        self._K_1_59375 = FixedPoint.quantize(1.59375, value_bits, frac_bits)
 
         # The limit is to choose the PWL function is 1.5 (= 3 * 2^-1), that only containts
         # 1 bit in int part. But, this comparison happens before to any mathematical operation,
         # so we need that self._limit_1_5.frac_bits = x.frac_bits to avoid implicit aligmnments
-        self._limit_1_5 = FixedPoint.quantize(1.5, frac_bits - mul_frac_bits, value_bits)
+        self._limit_1_5 = FixedPoint.quantize(1.5, value_bits, frac_bits - mul_frac_bits)
 
         # We need to store the input frac bits to reproduce the reciprocal on hardware.
         self.input_frac_bits = CalibrableVariable()
