@@ -18,7 +18,8 @@ import tensorflow as tf
 import keras
 
 from ..tensors import FixedPoint, floor_through, round_log2, MAX_BUFFER_BITWIDTH, pow2
-from .layers import deserialize_quant_object, Calibrable, CalibrableVariable
+from .calibrable import Calibrable, CalibrableVariable
+from .quantizers import deserialize_quantizer, OutputQuantizer
 
 
 __all__ = ["shiftmax", "Shiftmax", "QuantizedShiftmax"]
@@ -30,12 +31,12 @@ def shiftmax(logits, axis=-1):
     Used as approximation of the softmax activation.
 
     This function performs the equivalent of
-    ```python
-    logits = tf.floor(logits)
-    exp = 2 ** logits
-    sum_exp_shift = tf.round(tf.log2(tf.reduce_sum(exp, axis, keepdims=True)))
-    softmax = exp / 2 ** sum_exp_shift = 2 ** (logits -  sum_exp_shift)
-    ```
+
+        >>>  logits = tf.floor(logits)
+        >>>  exp = 2 ** logits
+        >>>  sum_exp_shift = tf.round(tf.log2(tf.reduce_sum(exp, axis, keepdims=True)))
+        >>>  softmax = exp / 2 ** sum_exp_shift = 2 ** (logits -  sum_exp_shift)
+
 
     When 2 ** :attr:`sum_exp_shift` is an approximated of sum_exp as a Power-of-Two (PoT)
 
@@ -45,8 +46,12 @@ def shiftmax(logits, axis=-1):
     Making the variable change :math:`y=logits-x0`, we reach the same result as
     :math:`p=shiftmax(logits)`, because,
 
-    p' = 2^y/sum(2^y) = 2^(logits-x0)/sum(2^(logits-x0))
-       = (2^logits * 2^-x0)/(2^-x0 * sum(2^logits)) = 2^logits/sum(2^logits) = p
+    .. math::
+        p' = \\frac{2^y}{sum(2^y)}
+           = \\frac{2^{logits-x0}}{sum(2^{logits-x0})}
+           = \\frac{2^{logits} * 2^{-x0}}{2^{-x0} * sum(2^{logits})}
+           = \\frac{2^{logits}}{sum(2^{logits})}
+           = p
 
     We take :math:`x0 = max(logits)`.
 
@@ -56,8 +61,7 @@ def shiftmax(logits, axis=-1):
             on. The default is -1 which indicates the last dimension.
 
     Returns:
-        :obj:`tf.Tensor` value of shiftmax function with the same type and
-            shape as `logits`.
+        tf.Tensor: value of shiftmax function with the same type and shape as `logits`.
 
     Raises:
         InvalidArgumentError: if `logits` is empty or `axis` is beyond the last
@@ -96,17 +100,17 @@ class QuantizedShiftmax(Calibrable, keras.layers.Layer):
     """A quantized layer to do a quantized function similar to the softmax, but
     using base 2 instead of e. So we replace
 
-        softmax(x_i) = e^x_i / sum(e^x_k)
+    .. math:: softmax(x_i) = \\frac{e^{x_i}}{sum(e^{x_k})}
 
     With this:
 
-        softmax2(x_i) = 2^x_i / sum(2^x_k)
+    .. math:: softmax2(x_i) = \\frac{2^{x_i}}{sum(2^{x_k})}
 
     This approximation is close enough to the original function. In order to
-    make it more hardware friendly, we also approximated the sum(2^x_k) to the
-    closest power of two:
+    make it more hardware friendly, we also approximated the :math:`sum(2^{x_k})`
+    to the closest power of two:
 
-        shiftmax(x_i) = 2^x_i / 2^round(log2(sum(2^x_k)))
+    .. math:: shiftmax(x_i) = \\frac{2^{x_i}}{2^{round(log2(sum(2^{x_k})))}}
 
     So it can be implemented with a simple shift operation.
 
@@ -120,8 +124,8 @@ class QuantizedShiftmax(Calibrable, keras.layers.Layer):
     def __init__(self, *args, quant_config={}, **kwargs):
         super().__init__(*args, **kwargs)
         self.quant_config = quant_config
-        self.out_quantizer = deserialize_quant_object(
-            self.quant_config, "output_quantizer", False)
+        self.out_quantizer = deserialize_quantizer(
+            self.quant_config, "output_quantizer", OutputQuantizer, False)
         self.buffer_bitwidth = self.quant_config.get("buffer_bitwidth", MAX_BUFFER_BITWIDTH) - 1
         self.exp_bitwidth = self.quant_config.get("exp_bitwidth", 8)
         if self.buffer_bitwidth <= 2 * self.exp_bitwidth:
@@ -139,7 +143,7 @@ class QuantizedShiftmax(Calibrable, keras.layers.Layer):
         if isinstance(x, tf.Tensor):
             # Assume the inputs are integer stored as float, which is the only
             # tf.Tensor inputs that are allowed
-            x = FixedPoint.quantize(x, 0, 8)
+            x = FixedPoint(x, 0, 8)
 
         # To avoid overflowing, some modifications are made to the input.
         # First remove the fractional part of the input (floor(x)). We can do
