@@ -17,7 +17,8 @@
 import keras
 import tensorflow as tf
 
-from .layers import deserialize_quant_object, Calibrable, CalibrableVariable
+from .calibrable import Calibrable, CalibrableVariable
+from .quantizers import deserialize_quantizer, WeightQuantizer, OutputQuantizer
 from ..tensors import FixedPoint, MAX_BUFFER_BITWIDTH
 
 
@@ -74,15 +75,19 @@ class AddPositionEmbs(keras.layers.Layer):
 @tf.keras.utils.register_keras_serializable()
 class QuantizedClassToken(Calibrable, ClassToken):
     """Quantize the :class:`ClassToken` layer, allowing quantization of the output.
+
+    Args:
+        quant_config (dict, optional): the serialized quantization configuration.
+            Defaults to empty configuration.
     """
 
     def __init__(self, *args, quant_config={}, **kwargs):
         super().__init__(*args, **kwargs)
         self.quant_config = quant_config
-        self.out_quantizer = deserialize_quant_object(
-            self.quant_config, "output_quantizer", False)
-        self.cls_quantizer = deserialize_quant_object(
-            self.quant_config, "cls_quantizer", True)
+        self.out_quantizer = deserialize_quantizer(
+            self.quant_config, "output_quantizer", OutputQuantizer, False)
+        self.cls_quantizer = deserialize_quantizer(
+            self.quant_config, "cls_quantizer", WeightQuantizer, True)
         self.buffer_bitwidth = self.quant_config.get(
             "buffer_bitwidth", MAX_BUFFER_BITWIDTH) - 1
 
@@ -125,15 +130,19 @@ class QuantizedClassToken(Calibrable, ClassToken):
 @tf.keras.utils.register_keras_serializable()
 class QuantizedAddPositionEmbs(Calibrable, AddPositionEmbs):
     """Quantize the :class:`AddPositionEmbs` layer, allowing operations in FixedPoint domain.
+
+    Args:
+        quant_config (dict, optional): the serialized quantization configuration.
+            Defaults to empty configuration.
     """
 
     def __init__(self, *args, quant_config={}, **kwargs):
         super().__init__(*args, **kwargs)
         self.quant_config = quant_config
-        self.out_quantizer = deserialize_quant_object(
-            self.quant_config, "output_quantizer", False)
-        self.pe_quantizer = deserialize_quant_object(
-            self.quant_config, "pe_quantizer", True)
+        self.out_quantizer = deserialize_quantizer(
+            self.quant_config, "output_quantizer", OutputQuantizer, False)
+        self.pe_quantizer = deserialize_quantizer(
+            self.quant_config, "pe_quantizer", WeightQuantizer, True)
         self.buffer_bitwidth = self.quant_config.get(
             "buffer_bitwidth", MAX_BUFFER_BITWIDTH) - 1
 
