@@ -27,7 +27,7 @@ from ..layers import (QuantizedDense, QuantizedLayerNormalization, QuantizedAddP
                       QuantizedReshape, QuantizedAttention, QuantizedConv2D, QuantizedMaxPool2D,
                       QuantizedGlobalAveragePooling2D, QuantizedExtractToken, Dequantizer,
                       Quantizer, QuantizedSeparableConv2D, QuantizedFlatten, QuantizedReciprocal,
-                      FixedPointQuantizer, WeightQuantizer, QuantizedConv2DTranspose,
+                      OutputQuantizer, WeightQuantizer, QuantizedConv2DTranspose,
                       QuantizedShiftmax, QuantizedSeparableConv2DTranspose)
 
 # Mapper to match float layer with its quantized version
@@ -44,7 +44,7 @@ CUSTOM_QLAYERS = {
     "SeparableConv2DTranspose": QuantizedSeparableConv2DTranspose
 }
 # List of Quantizer layer's that do not have a float layer representation
-NO_FLOAT_CUSTOM_QLAYERS = [Dequantizer, FixedPointQuantizer, WeightQuantizer]
+NO_FLOAT_CUSTOM_QLAYERS = [Dequantizer, OutputQuantizer, WeightQuantizer]
 
 
 def _handle_not_quantizable_layers(model):
@@ -101,6 +101,7 @@ def quantize(model, q_config, add_dequantizer=True):
        the "Transformer/EncoderBlock_5/MlpBlock/activation" key.
 
        Notes:
+
             * If configuration for all blocks are not defined, a global configuration is required.
             * To avoid ambiguity, the key with the most details in terms of hierarchy should be
               selected, e.g. "activation" key in the quantization configuration will be enough to
@@ -115,6 +116,7 @@ def quantize(model, q_config, add_dequantizer=True):
        "EncoderBlock_1/add_0", "EncoderBlock_1/add_1", ...).
 
         Note:
+
             The configuration priority in this case is defined by the number of hierarchical terms
             and the detail in subindices from the end to the beginning of the key name. That is
             "EncoderBlock/add_0" will have priority over "EncoderBlock/add" and
@@ -136,7 +138,7 @@ def quantize(model, q_config, add_dequantizer=True):
     #   - identifying items with the '/' separator,
     #   - replace trailing <_N> in each item by a matching pattern,
     #   - concatenate back items matching patterns.
-    q_config_regex = {re.compile("/" + "(?:|_[\d]+)/".join(key[1:].split('/')) + "(?:|_[\d]+)$"):
+    q_config_regex = {re.compile("/" + r"(?:|_[\d]+)/".join(key[1:].split('/')) + r"(?:|_[\d]+)$"):
                       (key[1:], obj) for key, obj in q_config.items()}
 
     # Layers will no longer be quantized as soon as a Dequantizer is found
