@@ -18,6 +18,7 @@
 Common utility methods used in quantization models.
 """
 
+from copy import deepcopy
 from pathlib import Path
 import numpy as np
 import tensorflow_addons as tfa
@@ -129,21 +130,38 @@ def insert_layer(model, target_layer_name, new_layer):
     insert_before = isinstance(new_layer, Quantizer)
     for layer in clone_model.layers[1:]:
         # Determine input tensors
-        layer_input = [network_dict['new_output_tensor_of'][layer_aux]
-                       for layer_aux in network_dict['input_layers_of'][layer.name]]
+        layer_input = []
+        for layer_aux in network_dict['input_layers_of'][layer.name]:
+            input = network_dict['new_output_tensor_of'][layer_aux]
+            if isinstance(input, tuple):
+                network_dict['new_output_tensor_of'][layer_aux] = input[1:]
+                input = input[0]
+            layer_input.append(input)
         if len(layer_input) == 1:
             layer_input = layer_input[0]
 
         # Insert layer if name matches the given layer name
         if layer.name == target_layer_name:
             if insert_before:
-                # Apply new layer then current layer
-                x = new_layer(layer_input)
+                # Apply one Quantizer by input then current layer
+                if isinstance(layer_input, list):
+                    inputs = []
+                    for i in range(len(layer_input)):
+                        copied_layer = deepcopy(new_layer)
+                        copied_layer._name += "_" + str(i)
+                        y = copied_layer(layer_input[i])
+                        inputs.append(y)
+                    x = inputs
+                else:
+                    x = new_layer(layer_input)
                 x = layer(x)
             else:
                 # Apply current layer then new layer
                 x = layer(layer_input)
-                x = new_layer(x)
+                if isinstance(x, tuple):
+                    x = tuple(new_layer(i) for i in x)
+                else:
+                    x = new_layer(x)
         else:
             x = layer(layer_input)
 
