@@ -16,8 +16,8 @@
 # ******************************************************************************
 import tensorflow as tf
 
-from .qtensor import QTensor, round_through, floor_through, pow2
-from ..debugging import assert_equal, assert_less, assert_less_equal
+from .qtensor import QTensor, round_through, floor_through, pow2, ceil_log2
+from ..debugging import assert_equal, assert_none_equal, assert_less, assert_less_equal
 
 
 class FixedPoint(QTensor):
@@ -37,6 +37,7 @@ class FixedPoint(QTensor):
             self.frac_bits = tf.cast(frac_bits, tf.float32)
         else:
             self.frac_bits = tf.convert_to_tensor(frac_bits, tf.float32)
+        assert_less_equal(0, self.frac_bits, "The frac_bits must be positive.")
         self.value_bits = value_bits
         # We store integer values in a float tensor to speed up calculations
         if isinstance(values, tf.Tensor):
@@ -52,26 +53,50 @@ class FixedPoint(QTensor):
         return tf.rank(self.frac_bits) == 0
 
     @staticmethod
-    def quantize(x, frac_bits, value_bits):
-        """Converts a float Tensor to a FixedPoint
+    def quantize(x, value_bits, frac_bits=None):
+        r"""Converts a float Tensor to a FixedPoint
 
         It converts the original float values into integer values so that:
 
-        x_int = round(x * 2 ^ frac_bits)
+        .. math:: {x_{int}} = round(x * 2^{frac\_bits})
 
-        It then evaluates the maximum integer values that can be stored for
-        the specified value bits: int_max = 2^bits - 1.
+        Note: 2^-frac_bits represents the FixedPoint precision.
 
-        The resulting integer values are clipped to [-int_max, int_max].
+        Before returning, the resulting integer values are clipped to the
+        maximum integer values that can be stored for the specified value bits:
+
+            [-2^value_bits, 2^value_bits - 1]
+
+        If frac_bits is not specified, the method starts by evaluating the number
+        of bits to dedicate to represent the integer part of the float tensor,
+        clipped to value_bits:
+
+        .. math:: int_bits = ceil(log2(x))
+
+        Note: this number can be negative when x < 1.
+
+        It then evaluates the offset of the least significant bit of the fractional
+        part of the float starting from zero. This represents the fractional bits:
+
+        .. math:: frac_bits = value_bits - int_bits
 
         Args:
             x (:obj:`tensorflow.Tensor`): a tensor of float values.
-            frac_bits (:obj:`tensorflow.Tensor`): an integer tensor of fractional bits
             value_bits (int): the number of value bits
+            frac_bits (:obj:`tensorflow.Tensor`, optional): an integer tensor of fractional bits.
+                Defaults to None.
 
         Returns:
             a :obj:`FixedPoint`
         """
+        if frac_bits is None:
+            if isinstance(x, int) or tf.reduce_all(x == tf.math.round(x)):
+                # The input does not need to be quantized
+                frac_bits = 0
+            else:
+                # Evaluate integer bits
+                int_bits = ceil_log2(tf.abs(x))
+                frac_bits = tf.maximum(0, value_bits - int_bits)
         # Project float into integer fixed-point space
         values = round_through(x * pow2(frac_bits))
         return FixedPoint(values, frac_bits, value_bits)
@@ -111,6 +136,7 @@ class FixedPoint(QTensor):
 
         This returns an equivalent FixedPoint with a scalar fractional bit
         corresponding to the maximum of:
+
             - the initial fractional bits on all channels,
             - when specified, the other FixedPoint fractional bits on all channels.
 
@@ -165,7 +191,7 @@ class FixedPoint(QTensor):
         # positive/negative shift PoT:
         # - for positive shift, the results are integer and can safely be floored,
         # - for negative shift, flooring simulates a right_shift.
-        values = floor_through(self.values * pow2(shift))
+        values = round_through(self.values * pow2(shift))
         # return a new FixedPoint with the target precision and bitwidth
         return FixedPoint(values, frac_bits, value_bits), shift
 
@@ -182,7 +208,9 @@ class FixedPoint(QTensor):
 
         This takes a tensor of shift values and apply them on each item of the
         FixedPoint values.
+
         The shift values should positive or negative integer:
+
         - if the value is positive, it is a left-shift,
         - if the value is negative, it is a right-shift.
 
@@ -259,7 +287,7 @@ class FixedPoint(QTensor):
         if isinstance(other, int):
             # Convert integer into a 32-bit fixed-point with no fractional bits,
             # aligned with the current FixedPoint
-            return self + FixedPoint.quantize(other, self.frac_bits, value_bits=32)
+            return self + FixedPoint.quantize(other, 32, self.frac_bits)
         elif isinstance(other, FixedPoint):
             # Check that self and other are per-tensor
             self.assert_per_tensor()
@@ -275,7 +303,7 @@ class FixedPoint(QTensor):
         if isinstance(other, int):
             # Convert integer into a 32-bit fixed-point with no fractional bits,
             # aligned with the current FixedPoint
-            return self - FixedPoint.quantize(other, self.frac_bits, value_bits=32)
+            return self - FixedPoint.quantize(other, 32, self.frac_bits)
         elif isinstance(other, FixedPoint):
             # Check that self and other are per-tensor
             self.assert_per_tensor()
@@ -300,8 +328,13 @@ class FixedPoint(QTensor):
 
     def __truediv__(self, other):
         if isinstance(other, int):
-            d_values = floor_through(self.values / other)
-            return FixedPoint(d_values, self.frac_bits, self.value_bits)
+            return self / FixedPoint(other)
+        elif isinstance(other, FixedPoint):
+            assert_none_equal(other.values, 0.0, "Cannot divide by 0.")
+            # The division between fixed-point is straight-forward
+            d_values = floor_through(self.values / other.values)
+            # Return a new FixedPoint whose frac bits from other is subtracted.
+            return FixedPoint(d_values, self.frac_bits - other.frac_bits, self.value_bits)
         raise TypeError(
             f"Unsupported operand type(s) for /: 'FixedPoint' and '{type(other)}'")
 
