@@ -44,14 +44,14 @@ class QFloat(QTensor):
 
     @staticmethod
     def quantize(x, float_max, value_bits, scales_bits):
-        """Converts a float Tensor to a QFloat
+        r"""Converts a float Tensor to a QFloat
 
         It first evaluates the maximum integer values that can be stored for
         the specified value bits: int_max = 2^bits - 1.
 
         It then converts the original float values into integer values so that:
 
-        x_int = round(x * int_max / float_max)
+        .. math:: {x_{int}} = round(x * \\frac{int\_max}{float\_max})
 
         The resulting integer values are clipped to [-int_max-1, int_max].
 
@@ -66,14 +66,8 @@ class QFloat(QTensor):
         """
         # Evaluate QFloat scales
         scales = tf.cast(float_max, tf.float32) / QTensor.int_max(value_bits)
-        # Make sure the scales are at least equal to the scales resolution,
-        # otherwise they would be quantized to zero
-        scales_res = pow2(-scales_bits)
-        scales = tf.math.maximum(scales, scales_res)
-        # Quantize the scales to a FixedPoint
-        scales_int_bits = tf.math.maximum(0., ceil_log2(scales))
-        scales_frac_bits = tf.clip_by_value(scales_bits - scales_int_bits,
-                                            0, scales_bits)
+        scales_int_bits = ceil_log2(scales)
+        scales_frac_bits = scales_bits - scales_int_bits
         # We do not apply the default FixedPoint quantization algorithm, as it
         # uses a round operation, and we want to make sure the quantized scales
         # are at least equal to the target scales. Should they be lower, this
